#
# Set minimum Terraform version and Terraform Cloud backend
#
terraform {
  required_version = ">= 0.12"
  // backend "http" {
  // }
}
/*
# Create local
*/
locals {
  prefix = "${var.project}-${var.environment}"
}
/*
# Create a random id
*/
resource "random_id" "id" {
  byte_length = 2
}
/*
# Create random password for BIG-IP
*/
resource "random_password" "password" {
  length           = 16
  special          = true
  override_special = "_%@"
}
/*
# Create Secret Store and Store BIG-IP Password
*/
resource "aws_secretsmanager_secret" "bigip" {
  name = format("%s-bigip-secret-%s", local.prefix, random_id.id.hex)
}
resource "aws_secretsmanager_secret_version" "bigip-pwd" {
  secret_id     = aws_secretsmanager_secret.bigip.id
  secret_string = random_password.password.result
}
/*
# Create VPC as per requirements
*/
module "vpc" {
  source = "../modules/services/network"

  providers = {
    aws = aws.secops
  }

  prefix = "${var.project}-${var.environment}"
  cidr   = var.cidr
  azs    = var.azs
  env    = var.environment
  random = random_id.id

}
/*
# Create BIG-IP host as per requirements
*/
module "bigip" {
  source = "../modules/functions/bigip"
  count = var.instance_count

  providers = {
    aws = aws.secops
  }

  prefix           = local.prefix
  cidr             = var.cidr
  azs              = var.azs
  env              = var.environment
  vpcid            = module.vpc.vpc_id
  public_subnets   = module.vpc.public_subnets
  private_subnets  = module.vpc.private_subnets
  database_subnets = module.vpc.database_subnets
  random           = random_id.id
  keyname          = var.ec2_key_name
  keyfile          = var.ec2_key_file
}
/*
# Create Docker host as per requirements
*/
module "docker" {
  source = "../modules/functions/docker"

  providers = {
    aws = aws.secops
  }

  prefix          = "${var.project}-${var.environment}"
  cidr            = var.cidr
  azs             = var.azs
  env             = var.environment
  vpcid           = module.vpc.vpc_id
  private_subnets = module.vpc.private_subnets
  random          = random_id.id
  keyname         = var.ec2_key_name
  keyfile         = var.ec2_key_file
}
/*
# Create Jump host as per requirements
*/
module "jumphost" {
  source = "../modules/functions/jumphost"

  providers = {
    aws = aws.secops
  }

  prefix            = "${var.project}-${var.environment}"
  region            = var.region
  cidr              = var.cidr
  azs               = var.azs
  env               = var.environment
  vpcid             = module.vpc.vpc_id
  public_subnets    = module.vpc.public_subnets
  random            = random_id.id
  keyname           = var.ec2_key_name
  keyfile           = var.ec2_key_file
}

module "ansible" {
  source = "../modules/functions/ansible"

  depends_on = [ module.bigip ]

  providers = {
    aws = aws.secops
  }

  prefix            = "${var.project}-${var.environment}"
  region            = var.region
  cidr              = var.cidr
  azs               = var.azs
  env               = var.environment
  vpcid             = module.vpc.vpc_id
  public_subnets    = module.vpc.public_subnets
  public_nic_ids = flatten(module.bigip[*].bigip_nic_ids["public_private"])
  docker_private_ip = module.docker.docker_private_ip
  random            = random_id.id
  keyname           = var.ec2_key_name
  keyfile           = var.ec2_key_file
  bigip_mgmt_addr   = flatten(module.bigip[*].mgmt_public_ips)
  bigip_mgmt_dns    = flatten(module.bigip[*].mgmt_public_dns)
  bigip_password    = flatten(module.bigip[*].bigip_password)
  bigip_private_add = flatten([for i in range(length(module.bigip.*.private_addresses)) : module.bigip.*.private_addresses[i]["public_private"]["private_ip"]])
  remote = module.jumphost.public_ip
}

